#ifndef __FTPCONNECTION__
#define __FTPCONNECTION__

#include <qstring.h>
#include <qthread.h>
#include <qsocket.h>
#include <qiodevice.h>
#include <qstringlist.h>

#include "FTPRequestReplySocket.h"
#include "FTPDataConnection.h"

struct GoodRet
{
	char *command;
	int good_codes[4];
};

class FTPConnection : public QObject
{
    Q_OBJECT

	private:
		FTPRequestReplySocket *socket;
		FTPDataConnection data_connection;
		bool login_successful;
		bool busy;

		bool passive;
		QString data_address;
		Q_INT16 data_port;
		QIODevice *io_device;
		bool delete_io;
		
		QString last_command;
		QString last_reply;
		int last_reply_code;

	public:
		FTPConnection();
		~FTPConnection();

	public slots:
		bool connect(const QString &host, Q_INT16 port, const QString &user, const QString &pass);
		void disconnect();
		void transfer_complete();

		void change_directory(const QString &dir);
		void change_parent_directory();
		void create_directory(const QString &dir);
		void delete_directory(const QString &dir);
		QString current_directory();
		
		void delete_file(const QString &file);

		void set_type_ascii();
		void set_type_binary();

		void set_passive();
		void set_port(Q_INT16);

		void send_noop();
		
	signals:
		void server_message(const QString &msg);
		void command_message(const QString &msg);
		void status_message(const QString &msg);
		void error(const QString &msg);

	public:
		bool connected() { return login_successful; }

		void get_file(const QString &file);
		void put_file(const QString &file);
    	QStringList dir_list(bool full=false);

		void wait_for_transfer()  { data_connection.wait(); }
		
	private:
		void send_message(const QString &msg);
		void recv_message();
		QString get_path();
		bool verify_return_code(const QString &command);
		bool issue_command(const QString &command);
		bool issue_path_command(const QString &com, const QString &dir);
		void start_passive();
};

#endif // __FTPCONNECTION__
