#include "FTPDataConnection.h"
#include <iostream>
using namespace std;

FTPDataConnection::FTPDataConnection()
{
	socket = NULL;
	device = NULL;
	mode = Stream;
	type = ASCII;
	send = false;
}

FTPDataConnection::~FTPDataConnection()
{
	if(socket != NULL)
	{
		socket->close();
		delete socket;
		socket = NULL;
		device = NULL;
	}
}

void FTPDataConnection::set_iodevice(QIODevice *d)
{
	device = d;
}

bool FTPDataConnection::establish_connection(const QString &addr, Q_INT16 port)
{
    QHostAddress a;
    if(!a.setAddress(addr))
	        return false;

    socket = new QSocketDevice();
    return socket->connect(a, port);
}

bool FTPDataConnection::establish_connection(Q_INT16 port)
{
	// Temporary hack until i can do this right
	QHostAddress a;
#ifdef LOCAL
	if(!a.setAddress("192.168.1.1"))
		return false;
#else
	if(!a.setAddress("131.215.84.128"))
		return false;
#endif
		
    QSocketDevice s;
	if(!s.bind(a,port))
	    return false;
	
	if(!s.listen(1))
		return false;

	int sock = s.accept();
	if(sock == -1)
	    return false;

	socket = new QSocketDevice(sock,QSocketDevice::Stream);
	
    return true;
}

void FTPDataConnection::set_mode(Mode m)
{
	mode = m;
}

void FTPDataConnection::set_type(Type t)
{
	type = t;
}

void FTPDataConnection::set_direction(bool s)
{
	send = s;
}

bool FTPDataConnection::is_open()
{
	if(socket != NULL)
		return socket->isOpen();
	return false;
}

void FTPDataConnection::run()
{
	if(send)
		send_data();
	else
		recv_data();

	if(!socket->isOpen())
	{
		socket->close();
		delete socket;
		socket = NULL;
	}
	
	emit transfer_complete();
}

void FTPDataConnection::recv_data()
{
	char *data;
	int data_length;

#ifdef DEBUG
	int total_length = 0;
	cout << "RECEIVING" << endl;
#endif
	
	while(!(socket->atEnd()))
	{
		switch(mode)
		{
			case Stream:
				data = new char[BLOCK_SIZE];
				data_length = socket->readBlock(data, BLOCK_SIZE);
				#ifdef DEBUG
					total_length += data_length;
					cout << "BYTES: " << total_length << endl;
				#endif
				break;
	
			case Block:
				break;
	
			case Compressed:
				break;
		}

		switch(type)
		{
			case ASCII:
				for(int i = 0; i < data_length; ++i)
				{
					if(data[i] == '\r')
						if(data[i+1] == '\n')
						{
							device->writeBlock("\n",1);
							++i;
							continue;
						}

					device->writeBlock(data+i,1);
				}
				break;
	
			case EBCDIC:
				break;
	
			case Image:
				device->writeBlock(data, data_length);
				break;
		}
		
		delete data;
	}
}

void FTPDataConnection::send_data()
{
	char *data;
	int data_length;
		
	while(!(device->atEnd()))
	{
		switch(type)
		{
			case ASCII:
				break;

			case EBCDIC:
				break;

			case Image:
				data = new char[BLOCK_SIZE];
				data_length = device->readBlock(data, BLOCK_SIZE);
				break;
		}

		switch(mode)
		{
			case Stream:
				socket->writeBlock(data, data_length);
				break;
					
			case Block:
				break;

			case Compressed:
				break;
		}

		delete data;
	}
}
